;;;; assert.scm - Some basic error-checking tools - felix
;
;
; REQUIRES: [srfi-23]
;           format
;
;
; [procedure] (assert TEST ARG ...)
;
;   If TEST evaulates to #f, signals an error with the string obtained by
;   evaluating (format ARG ...).
;
; [procedure] (ensure VAL PREDICATE [ARG ...]) -> VAL
;
;   Returns VAL, but signals an error, if (PREDICATE VAL) returns #f.
;   The error-message can optionally be given as a format string in ARG ...
;


(define (assert test . args)
  (if (not test)
      (error (apply format args)) ) )

(define (ensure value pred . args)
  (cond ((pred value) value)
	((null? args) (error "value is not if required type" value))
	(else (error (apply format args))) ) )
