;;;; match.scm - Simple pattern-matching procedure - felix
;
;
; REQUIRES: srfi-1
;           srfi-23
;
;
; [procedure] (match PATTERN EXPR SUCCESSPROC [FAILPROC])
;
;   Matches PATTERN with EXPR and invokes SUCCESSPROC with the values
;   of all matching variables of the form ,SYMBOL (or ,@SYMBOL). If the match
;   didn't succeed, FAILPROC is called (with zero arguments). If FAILPROC
;   is not supplied, an error is signalled.
;
;   PATTERN = ,SYMBOL
;           | (PATTERN ... [. PATTERN])
;           | #(PATTERN ...)
;           | EXPR
;


(define (match p x sk . fk)
  (let ((env '()))
    (define (mvar v x)
      (let ((a (assq v env)))
	(if a 
	    (equal? (cdr a) x)
	    (begin
	      (set! env (alist-cons v x env))
	      #t) ) ) )
    (define (walk p x)
      (if (vector? p) 
	  (and (vector? x)
	       (walk (vector->list p) (vector->list x)) )
	  (if (not-pair? p)
	      (equal? p x)
	      (let ((head (car p)))
		(if (eq? 'unquote head)
		    (mvar (cadr p) x)
		    (and (pair? x)
			 (walk head (car x))
			 (walk (cdr p) (cdr x)) ) ) ) ) ) )
    (cond ((walk p x) (apply sk (map cdr (reverse env))))
	  ((null? fk) (error "pattern does not match" p x))
	  (else ((car fk))) ) ) )
