#!/bin/bash
. ../path.inc
# preliminary stupid wrapper script

usage()
{
  cat 1>&2 <<EOF
Usage: $basename SRC_LANGUAGE SRC_STAGESET TRG_LANGUAGE TRG_STAGESET
   or: $basename -e STAGE_DESCRIPTION_FILE

Compiles from STDIN to STDOUT, from SRC_LANGUAGE to TRG_LANGUAGE,
with the selected stage sets used to specify the compilation process
in detail. The second type of usage expects a stage set given in
a user provided file.
EOF
  exit $1
}

failu()
{
  echo $1 1>&2
  usage 1
}

fail()
{
  echo $1 1>&2
  exit 1
}

failx()
{
  echo -n "//"
  fail $1
}

testpath()
{
  test "$1" != "." -a "$1" != ".." || failx "invalid partial path: $1"
  test -e "$2" || failx "path component not found: $2"
}

MLIB=$LIB/$NAME/l
MBIN=$BIN/$NAME

basename=$0
srclang=$1
srcstgs=$2
trglang=$3
trgstgs=$4

case "$1" in
  --help|-h*)
    usage 0
    ;;
  -e)
    test -z "$srcstgs" && failu "missing stageset"
    test -r "$srcstgs" || failu "file not found: $srcstgs"
    catstages="cat $srcstgs"
    ;;
  *)
    test -z "$srclang" && failu "missing source language"
    test -r $MLIB/$srclang || failu "unknown language: $srclang"
    test -z "$srcstgs" && failu "missing source stageset"
    test -r $MLIB/$srclang/a/$srcstgs || failu "unknown stageset: $srcstgs"
    test -z "$trglang" && failu "missing target language"
    test -r $MLIB/$trglang || failu "unknown language: $trglang"
    test -z "$trgstgs" && failu "missing target stageset"
    test -r $MLIB/$trglang/a/$trgstgs || failu "unknown stageset: $trgstgs"
    catstages="cat $MLIB/$srclang/a/$srcstgs $MLIB/$trglang/a/$trgstgs"
    ;;
esac

CMD=`eval $catstages | \
sed -e 's,#.*,,' | \
grep -v '^[[:space:]]*$' | \
while read stype a b c rest ; do
  case "$stype" in
    error)
      # error <stage>
      failx "not yet implemented: error"
      test -z "$b" || failx "extra parameters with error stage declamation"
      testpath $a $MBIN/c/$a
      ;;
    stage)
      # stage <stage> [<language> <table>]
      test -z "$rest" || failx "extra parameters with stage declamation"
      testpath $a $MBIN/c/$a
      if [ -z "$b" ]; then
        echo -n " | $MBIN/c/$a"
      else
        testpath $b $MLIB/$b
        testpath $c $MLIB/$b/d/$c
        echo -n " | $MBIN/c/$a $MLIB/$b/d/$c"
      fi
      ;;
    local)
      # local <language> <stage> ...
      testpath $a $MBIN/l/$a
      testpath $b $MBIN/l/$a/$b
      echo -n " | $MBIN/l/$a/$b $c $rest"
      ;;
    include)
      # include <stage> <language> <table>
      failx "not yet implemented: include"
      test -z "$rest" || failx "extra parameters with include stage declamation"
      testpath $a $MBIN/c/$a
      testpath $b $MLIB/$b
      testpath $c $MLIB/$b/d/$c
      ;;
    *)
      failx "unknown stage type: $stype"
      ;;
  esac
done`

echo $CMD | grep "//" >/dev/null && exit 1
CMD="cat $CMD"
export LD_LIBRARY_PATH=/usr/lib/mother/lib
eval "$CMD"

